package com.icing.dialer.activities

import android.database.Cursor
import android.os.Bundle
import android.provider.CallLog
import io.flutter.embedding.android.FlutterActivity
import io.flutter.embedding.engine.FlutterEngine
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import com.icing.dialer.KeystoreHelper
import com.icing.dialer.services.CallService

class MainActivity: FlutterActivity() {
    // Existing channel for keystore operations.
    private val KEYSTORE_CHANNEL = "com.example.keystore"
    // New channel for call log access.
    private val CALLLOG_CHANNEL = "com.example.calllog"
    
    private val CALL_CHANNEL = "call_service"

    override fun configureFlutterEngine(flutterEngine: FlutterEngine) {
        super.configureFlutterEngine(flutterEngine)

        // Call service channel
        MethodChannel(flutterEngine.dartExecutor.binaryMessenger, CALL_CHANNEL).setMethodCallHandler { call, result ->
            when (call.method) {
                "makeGsmCall" -> {
                    val phoneNumber = call.argument<String>("phoneNumber")
                    if (phoneNumber != null) {
                        CallService.makeGsmCall(this, phoneNumber)
                        result.success("Calling $phoneNumber")
                    } else {
                        result.error("INVALID_PHONE_NUMBER", "Phone number is required", null)
                    }
                }
                "hangUpCall" -> {
                    CallService.hangUpCall(this)
                    result.success("Call ended")
                }
                else -> result.notImplemented()
            }
        }

        // Set up the keystore channel.
        MethodChannel(flutterEngine.dartExecutor.binaryMessenger, KEYSTORE_CHANNEL)
            .setMethodCallHandler { call, result ->
                // Delegate method calls to KeystoreHelper.
                KeystoreHelper(call, result).handleMethodCall()
            }

        // Set up the call log channel.
        MethodChannel(flutterEngine.dartExecutor.binaryMessenger, CALLLOG_CHANNEL)
            .setMethodCallHandler { call, result ->
                if (call.method == "getCallLogs") {
                    val callLogs = getCallLogs()
                    result.success(callLogs)
                } else {
                    result.notImplemented()
                }
            }
    }

    /**
     * Queries the Android call log and returns a list of maps.
     * Each map contains keys: "number", "type", "date", and "duration".
     */
    private fun getCallLogs(): List<Map<String, Any?>> {
        val logsList = mutableListOf<Map<String, Any?>>()
        val cursor: Cursor? = contentResolver.query(
            CallLog.Calls.CONTENT_URI,
            null,
            null,
            null,
            CallLog.Calls.DATE + " DESC"
        )
        if (cursor != null) {
            while (cursor.moveToNext()) {
                val number = cursor.getString(cursor.getColumnIndexOrThrow(CallLog.Calls.NUMBER))
                val type = cursor.getInt(cursor.getColumnIndexOrThrow(CallLog.Calls.TYPE))
                val date = cursor.getLong(cursor.getColumnIndexOrThrow(CallLog.Calls.DATE))
                val duration = cursor.getLong(cursor.getColumnIndexOrThrow(CallLog.Calls.DURATION))

                val map = HashMap<String, Any?>()
                map["number"] = number
                map["type"] = type  // Typically: 1 for incoming, 2 for outgoing, 3 for missed.
                map["date"] = date
                map["duration"] = duration
                logsList.add(map)
            }
            cursor.close()
        }
        return logsList
    }
}
